# Generated by roxygen2: do not edit by hand

S3method("na_range<-",data.frame)
S3method("na_range<-",default)
S3method("na_range<-",factor)
S3method("na_range<-",haven_labelled)
S3method("na_range<-",survey.design)
S3method("na_range<-",svyrep.design)
S3method("na_values<-",data.frame)
S3method("na_values<-",default)
S3method("na_values<-",factor)
S3method("na_values<-",haven_labelled)
S3method("na_values<-",survey.design)
S3method("na_values<-",svyrep.design)
S3method("val_label<-",character)
S3method("val_label<-",data.frame)
S3method("val_label<-",default)
S3method("val_label<-",haven_labelled)
S3method("val_label<-",numeric)
S3method("val_label<-",survey.design)
S3method("val_label<-",svyrep.design)
S3method("val_labels<-",character)
S3method("val_labels<-",data.frame)
S3method("val_labels<-",default)
S3method("val_labels<-",factor)
S3method("val_labels<-",haven_labelled)
S3method("val_labels<-",haven_labelled_spss)
S3method("val_labels<-",numeric)
S3method("val_labels<-",survey.design)
S3method("val_labels<-",svyrep.design)
S3method("var_label<-",data.frame)
S3method("var_label<-",default)
S3method("var_label<-",survey.design)
S3method("var_label<-",svyrep.design)
S3method(copy_labels,data.frame)
S3method(copy_labels,default)
S3method(copy_labels,haven_labelled)
S3method(copy_labels,survey.design)
S3method(copy_labels,svyrep.design)
S3method(drop_unused_value_labels,data.frame)
S3method(drop_unused_value_labels,default)
S3method(drop_unused_value_labels,haven_labelled)
S3method(drop_unused_value_labels,survey.design)
S3method(drop_unused_value_labels,svyrep.design)
S3method(na_range,data.frame)
S3method(na_range,default)
S3method(na_range,haven_labelled_spss)
S3method(na_range,survey.design)
S3method(na_range,svyrep.design)
S3method(na_values,data.frame)
S3method(na_values,default)
S3method(na_values,haven_labelled_spss)
S3method(na_values,survey.design)
S3method(na_values,svyrep.design)
S3method(names_prefixed_by_values,default)
S3method(names_prefixed_by_values,list)
S3method(nolabel_to_na,data.frame)
S3method(nolabel_to_na,default)
S3method(nolabel_to_na,haven_labelled)
S3method(nolabel_to_na,survey.design)
S3method(nolabel_to_na,svyrep.design)
S3method(print,look_for)
S3method(recode,haven_labelled)
S3method(remove_attributes,data.frame)
S3method(remove_attributes,default)
S3method(remove_attributes,survey.design)
S3method(remove_attributes,svyrep.design)
S3method(remove_labels,data.frame)
S3method(remove_labels,default)
S3method(remove_labels,haven_labelled_spss)
S3method(remove_labels,survey.design)
S3method(remove_labels,svyrep.design)
S3method(remove_user_na,data.frame)
S3method(remove_user_na,default)
S3method(remove_user_na,haven_labelled_spss)
S3method(remove_user_na,survey.design)
S3method(remove_user_na,svyrep.design)
S3method(remove_val_labels,data.frame)
S3method(remove_val_labels,default)
S3method(remove_val_labels,survey.design)
S3method(remove_val_labels,svyrep.design)
S3method(remove_var_label,data.frame)
S3method(remove_var_label,default)
S3method(remove_var_label,survey.design)
S3method(remove_var_label,svyrep.design)
S3method(sort_val_labels,data.frame)
S3method(sort_val_labels,default)
S3method(sort_val_labels,haven_labelled)
S3method(sort_val_labels,survey.design)
S3method(sort_val_labels,svyrep.design)
S3method(tagged_na_to_regular_na,data.frame)
S3method(tagged_na_to_regular_na,default)
S3method(tagged_na_to_regular_na,double)
S3method(tagged_na_to_regular_na,survey.design)
S3method(tagged_na_to_regular_na,svyrep.design)
S3method(tagged_na_to_user_na,data.frame)
S3method(tagged_na_to_user_na,default)
S3method(tagged_na_to_user_na,double)
S3method(tagged_na_to_user_na,survey.design)
S3method(tagged_na_to_user_na,svyrep.design)
S3method(to_character,data.frame)
S3method(to_character,default)
S3method(to_character,double)
S3method(to_character,haven_labelled)
S3method(to_character,survey.design)
S3method(to_character,svyrep.design)
S3method(to_factor,data.frame)
S3method(to_factor,default)
S3method(to_factor,factor)
S3method(to_factor,haven_labelled)
S3method(to_factor,survey.design)
S3method(to_factor,svyrep.design)
S3method(to_labelled,data.frame)
S3method(to_labelled,data.set)
S3method(to_labelled,factor)
S3method(to_labelled,importer)
S3method(to_labelled,list)
S3method(update_labelled,data.frame)
S3method(update_labelled,default)
S3method(update_labelled,haven_labelled)
S3method(update_labelled,haven_labelled_spss)
S3method(update_labelled,labelled)
S3method(update_value_labels_with,data.frame)
S3method(update_value_labels_with,survey.design)
S3method(update_value_labels_with,svyrep.design)
S3method(update_variable_labels_with,data.frame)
S3method(update_variable_labels_with,survey.design)
S3method(update_variable_labels_with,svyrep.design)
S3method(user_na_to_na,data.frame)
S3method(user_na_to_na,default)
S3method(user_na_to_na,haven_labelled_spss)
S3method(user_na_to_na,survey.design)
S3method(user_na_to_na,svyrep.design)
S3method(user_na_to_tagged_na,data.frame)
S3method(user_na_to_tagged_na,default)
S3method(user_na_to_tagged_na,haven_labelled_spss)
S3method(user_na_to_tagged_na,survey.design)
S3method(user_na_to_tagged_na,svyrep.design)
S3method(val_label,data.frame)
S3method(val_label,default)
S3method(val_label,haven_labelled)
S3method(val_label,survey.design)
S3method(val_label,svyrep.design)
S3method(val_labels,data.frame)
S3method(val_labels,default)
S3method(val_labels,haven_labelled)
S3method(val_labels,survey.design)
S3method(val_labels,svyrep.design)
S3method(val_labels_to_na,data.frame)
S3method(val_labels_to_na,default)
S3method(val_labels_to_na,haven_labelled)
S3method(val_labels_to_na,survey.design)
S3method(val_labels_to_na,svyrep.design)
S3method(var_label,data.frame)
S3method(var_label,default)
S3method(var_label,survey.design)
S3method(var_label,svyrep.design)
export("%>%")
export("label_attribute<-")
export("na_range<-")
export("na_values<-")
export("val_label<-")
export("val_labels<-")
export("var_label<-")
export(add_value_labels)
export(convert_list_columns_to_character)
export(copy_labels)
export(copy_labels_from)
export(dictionary_to_value_labels)
export(dictionary_to_variable_labels)
export(drop_unused_value_labels)
export(duplicated_tagged_na)
export(foreign_to_labelled)
export(format_tagged_na)
export(generate_dictionary)
export(get_label_attribute)
export(get_na_range)
export(get_na_values)
export(get_value_labels)
export(get_variable_labels)
export(is.labelled)
export(is_prefixed)
export(is_regular_na)
export(is_tagged_na)
export(is_user_na)
export(label_attribute)
export(labelled)
export(labelled_spss)
export(look_for)
export(look_for_and_select)
export(lookfor)
export(lookfor_to_long_format)
export(memisc_to_labelled)
export(na_range)
export(na_tag)
export(na_values)
export(names_prefixed_by_values)
export(nolabel_to_na)
export(order_tagged_na)
export(print_labels)
export(print_tagged_na)
export(recode_if)
export(remove_attributes)
export(remove_labels)
export(remove_user_na)
export(remove_val_labels)
export(remove_value_labels)
export(remove_var_label)
export(set_label_attribute)
export(set_na_range)
export(set_na_values)
export(set_value_labels)
export(set_variable_labels)
export(sort_tagged_na)
export(sort_val_labels)
export(tagged_na)
export(tagged_na_to_regular_na)
export(tagged_na_to_user_na)
export(to_character)
export(to_factor)
export(to_gt)
export(to_labelled)
export(unique_tagged_na)
export(unlabelled)
export(update_labelled)
export(update_value_labels_with)
export(update_variable_labels_with)
export(user_na_to_na)
export(user_na_to_regular_na)
export(user_na_to_tagged_na)
export(val_label)
export(val_labels)
export(val_labels_to_na)
export(var_label)
import(rlang)
importFrom(dplyr,`%>%`)
importFrom(dplyr,recode)
importFrom(dplyr,where)
importFrom(haven,format_tagged_na)
importFrom(haven,is.labelled)
importFrom(haven,is_tagged_na)
importFrom(haven,labelled)
importFrom(haven,labelled_spss)
importFrom(haven,na_tag)
importFrom(haven,print_labels)
importFrom(haven,print_tagged_na)
importFrom(haven,tagged_na)
importFrom(lifecycle,deprecate_soft)
