% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_prefixed.R
\name{is_prefixed}
\alias{is_prefixed}
\title{Check if a factor is prefixed}
\usage{
is_prefixed(x)
}
\arguments{
\item{x}{a factor}
}
\description{
Check if a factor is prefixed
}
